//
//  GameScene.swift
//  TowerDefense
//
//  Created by 杉本裕樹 on 2015/10/03.
//  Copyright (c) 2015年 MyName. All rights reserved.
//

import SpriteKit
import Foundation
import GameplayKit

class GameScene: SKScene, SKPhysicsContactDelegate {
    enum State {
        case Playing
        case GameClear
        case GameOver
    }
    var state = State.Playing
    let enemy = SKSpriteNode(imageNamed: "Enemy")
    let char = SKSpriteNode(imageNamed: "Char")
    
    override func didMoveToView(view: SKView) {
        physicsWorld.gravity = CGVectorMake(0, 0)
        physicsWorld.contactDelegate = self
        let fieldData = [
            [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
            [0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0],
            [0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0],
            [0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0],
            [0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0],
            [0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0],
            [0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0],
            [0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0],
            [0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0],
            [0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0],
            [0, 0, 1, 0, 0, 0, 1, 1, 1, 1, 1],
            [0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 0],
            [0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 0],
            [0, 0, 1, 1, 1, 1, 1, 0, 0, 0, 0],
            [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
            [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
            [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
            [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
            [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
            [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
        ]
        
        let fieldImageLength = view.frame.width / 10
        for (i, data) in fieldData.enumerate() {
            for (j, value) in data.enumerate() {
                let field = SKSpriteNode(imageNamed: "Field\(value)")
                field.name = "Field\(value)"
                field.size = CGSize(width: fieldImageLength, height: fieldImageLength)
                field.physicsBody = SKPhysicsBody(rectangleOfSize: field.size)
                field.physicsBody?.categoryBitMask = 0x0
                field.physicsBody?.collisionBitMask = 0x0
                field.position = CGPoint(
                    x: CGFloat(j) * fieldImageLength,
                    y: view.frame.height - CGFloat(i - 1) * fieldImageLength)
                field.zPosition = -1
                addChild(field)
            }
        }
        
        char.position = CGPoint(x:250, y:300)
        char.physicsBody = SKPhysicsBody(rectangleOfSize: char.size)
        char.physicsBody?.contactTestBitMask = 0x1
        addChild(char)
        
        let fields = children.filter { $0.name == "Field0" }
        let obstacles = SKNode.obstaclesFromNodePhysicsBodies(fields)
        let graph = GKObstacleGraph(obstacles: obstacles, bufferRadius: 10)
        let start = GKGraphNode2D(point: vector_float2(Float(fieldImageLength * 2), Float(view.frame.height)))
        let end = GKGraphNode2D(point: vector_float2(Float(fieldImageLength * 13), Float(view.frame.height - fieldImageLength * 9)))
        graph.connectNodeUsingObstacles(start)
        graph.connectNodeUsingObstacles(end)
        let nodes = graph.findPathFromNode(start, toNode: end)
        let actions = nodes.flatMap { $0 as? GKGraphNode2D }.map {
            SKAction.moveTo(CGPoint(x: Double($0.position.x), y: Double($0.position.y)), duration: 5)
        }
        
        enemy.position = CGPoint(x: fieldImageLength * 2, y: view.frame.height)
        enemy.physicsBody = SKPhysicsBody(rectangleOfSize: enemy.size)
        enemy.runAction(SKAction.sequence(actions)) {
            self.state = .GameOver
            
            let myLabel = SKLabelNode(fontNamed: "HiraginoSans-W6")
            myLabel.text = "ゲームオーバー"
            myLabel.fontSize = 45
            myLabel.position = CGPoint(x:CGRectGetMidX(self.frame), y:CGRectGetMidY(self.frame) - 20)
            self.addChild(myLabel)
        }
        addChild(enemy)
    }
    
    func didBeginContact(contact: SKPhysicsContact) {
        state = .GameClear
        enemy.removeFromParent()
        
        let myLabel = SKLabelNode(fontNamed: "HiraginoSans-W6")
        myLabel.text = "ゲームクリア"
        myLabel.fontSize = 45
        myLabel.position = CGPoint(x:CGRectGetMidX(self.frame), y:CGRectGetMidY(self.frame) - 20)
        addChild(myLabel)
    }
}
